@@ -496,10 +496,22 @@ class test_Channel(Case):
         c.parse_response = Mock()
         self.channel._poll_error('BRPOP')
 
-        c.parse_response.assert_called_with('BRPOP')
+        c.parse_response.assert_called_with(c.connection, 'BRPOP')
 
         c.parse_response.side_effect = KeyError('foo')
-        self.assertIsNone(self.channel._poll_error('BRPOP'))
+        with self.assertRaises(KeyError):
+            self.channel._poll_error('BRPOP')
+
+    def test_poll_error_on_type_LISTEN(self):
+        c = self.channel.subclient = Mock()
+        c.parse_response = Mock()
+        self.channel._poll_error('LISTEN')
+
+        c.parse_response.assert_called_with()
+
+        c.parse_response.side_effect = KeyError('foo')
+        with self.assertRaises(KeyError):
+            self.channel._poll_error('LISTEN')
 
     def test_put_fanout(self):
         self.channel._in_poll = False
