@@ -10,6 +10,7 @@ from __future__ import absolute_import
 import socket
 
 from bisect import bisect
+from collections import namedtuple
 from contextlib import contextmanager
 from time import time
 
@@ -42,7 +43,6 @@ try:
 except ImportError:  # pragma: no cover
     redis = None     # noqa
 
-
 from . import virtual
 
 logger = get_logger('kombu.transport.redis')
@@ -53,6 +53,10 @@ DEFAULT_DB = 0
 
 PRIORITY_STEPS = [0, 3, 6, 9]
 
+error_classes_t = namedtuple('error_classes_t', (
+    'connection_errors', 'channel_errors',
+))
+
 # This implementation may seem overly complex, but I assure you there is
 # a good reason for doing it this way.
 #
@@ -68,6 +72,28 @@ PRIORITY_STEPS = [0, 3, 6, 9]
 # queues manually.
 
 
+def get_redis_error_classes():
+    from redis import exceptions
+    # This exception suddenly changed name between redis-py versions
+    if hasattr(exceptions, 'InvalidData'):
+        DataError = exceptions.InvalidData
+    else:
+        DataError = exceptions.DataError
+    return error_classes_t(
+        (virtual.Transport.connection_errors + (
+            InconsistencyError,
+            socket.error,
+            IOError,
+            OSError,
+            exceptions.ConnectionError,
+            exceptions.AuthenticationError)),
+        (virtual.Transport.channel_errors + (
+            DataError,
+            exceptions.InvalidResponse,
+            exceptions.ResponseError)),
+    )
+
+
 class MutexHeld(Exception):
     pass
 
@@ -444,7 +470,12 @@ class Channel(virtual.Channel):
         except KeyError:
             return
         try:
-            self.active_fanout_queues.discard(queue)
+            self.active_fanout_queues.remove(queue)
+        except KeyError:
+            pass
+        else:
+            self._unsubscribe_from(queue)
+        try:
             self._fanout_to_queue.pop(self._fanout_queues[queue])
         except KeyError:
             pass
@@ -452,9 +483,12 @@ class Channel(virtual.Channel):
         self._update_cycle()
         return ret
 
+    def _get_subscribe_topic(self, queue):
+        return ''.join([self.keyprefix_fanout,
+                        self._fanout_queues[queue]])
+
     def _subscribe(self):
-        prefix = self.keyprefix_fanout
-        keys = [''.join([prefix, self._fanout_queues[queue]])
+        keys = [self._get_subscribe_topic(queue)
                 for queue in self.active_fanout_queues]
         if not keys:
             return
@@ -462,7 +496,20 @@ class Channel(virtual.Channel):
         if c.connection._sock is None:
             c.connection.connect()
         self._in_listen = True
-        self.subclient.subscribe(keys)
+        c.subscribe(keys)
+
+    def _unsubscribe_from(self, queue):
+        topic = self._get_subscribe_topic(queue)
+        c = self.subclient
+        should_disconnect = False
+        if c.connection._sock is None:
+            c.connection.connect()
+            should_disconnect = True
+        try:
+            c.unsubscribe([topic])
+        finally:
+            if should_disconnect:
+                c.connection.disconnect()
 
     def _handle_message(self, client, r):
         if r[0] == 'unsubscribe' and r[2] == 0:
@@ -828,22 +875,4 @@ class Transport(virtual.Transport):
 
     def _get_errors(self):
         """Utility to import redis-py's exceptions at runtime."""
-        from redis import exceptions
-        # This exception suddenly changed name between redis-py versions
-        if hasattr(exceptions, 'InvalidData'):
-            DataError = exceptions.InvalidData
-        else:
-            DataError = exceptions.DataError
-        return (
-            (virtual.Transport.connection_errors + (
-                InconsistencyError,
-                socket.error,
-                IOError,
-                OSError,
-                exceptions.ConnectionError,
-                exceptions.AuthenticationError)),
-            (virtual.Transport.channel_errors + (
-                DataError,
-                exceptions.InvalidResponse,
-                exceptions.ResponseError)),
-        )
+        return get_redis_error_classes()
