@@ -228,7 +228,7 @@ def retry_over_time(fun, catch, args=[], kwargs={}, errback=None,
         try:
             return fun(*args, **kwargs)
         except catch as exc:
-            if max_retries is not None and retries >= max_retries:
+            if max_retries and retries >= max_retries:
                 raise
             if callback:
                 callback()
@@ -407,7 +407,10 @@ class ChannelPromise(object):
             return value
 
     def __repr__(self):
-        return '<promise: %r>' % (self(), )
+        try:
+            return repr(self.__value__)
+        except AttributeError:
+            return '<promise: 0x{0:x}>'.format(id(self.__contract__))
 
 
 def escape_regex(p, white=''):
